window.Gleam.initSearch([{"doc":"wisp_kv_sessions","title":"wisp_kv_sessions","content":"# wisp_kv_sessions\n\n[![Package Version](https://img.shields.io/hexpm/v/wisp_kv_sessions)](https://hex.pm/packages/wisp_kv_sessions)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/wisp_kv_sessions/)\n\n\n# Overview\nwisp_kv_sessions is a key-value session management library for [Wisp](https://gleam-wisp.github.io/wisp/), inspired by the Rust crate [tower sessions](https://docs.rs/tower-sessions/latest/tower_sessions/#). This library allows you to manage user sessions with ease, storing session data in a simple key-value store. \n\n# Example Usage\nAn minimal example usage is available in `./example/app.gleam`.\n\n```gleam\nimport gleam/dynamic\nimport gleam/erlang/process\nimport gleam/json\nimport gleam/option\nimport gleam/result\nimport gleam/string_builder\nimport memory_store\nimport mist\nimport wisp\nimport wisp/wisp_mist\nimport wisp_kv_sessions\nimport wisp_kv_sessions/session\nimport wisp_kv_sessions/session_config\n\npub fn main() {\n  // Setup session_store\n  use memory_store &lt;- result.map(memory_store.try_create_session_store())\n\n  // Create session config\n  let session_config =\n    session_config.Config(\n      default_expiry: session.ExpireIn(60 * 60),\n      cookie_name: &quot;SESSION_COOKIE&quot;,\n      store: memory_store,\n    )\n\n  let secret_key_base = wisp.random_string(64)\n\n  // Start the Mist web server.\n  let assert Ok(_) =\n    wisp_mist.handler(handle_request(_, session_config), secret_key_base)\n    |&gt; mist.new\n    |&gt; mist.port(8000)\n    |&gt; mist.start_http\n\n  process.sleep_forever()\n}\n\npub fn handle_request(req: wisp.Request, session_config) -&gt; wisp.Response {\n  use req &lt;- wisp_kv_sessions.middleware(session_config, req)\n\n  case wisp.path_segments(req) {\n    [] -&gt; get_value_page(req, session_config)\n    [&quot;set&quot;] -&gt; set_value_page(req, session_config)\n    _ -&gt; wisp.not_found()\n  }\n}\n\nfn get_value_page(\n  req: wisp.Request,\n  session_config: session_config.Config,\n) -&gt; wisp.Response {\n  // Read value to the session\n  let assert Ok(key) =\n    wisp_kv_sessions.get(session_config, req, &quot;test_key&quot;, dynamic.string)\n\n  case key {\n    option.Some(k) -&gt; {\n      wisp.html_response(string_builder.from_string(k), 200)\n    }\n    option.None -&gt; {\n      wisp.html_response(\n        string_builder.from_string(&quot;No value set. Go to /set to set a value&quot;),\n        200,\n      )\n    }\n  }\n}\n\n// Not nessesary for this simple type \nfn encode_string(str: String) {\n  json.string(str) |&gt; json.to_string\n}\n\nfn set_value_page(\n  req: wisp.Request,\n  session_config: session_config.Config,\n) -&gt; wisp.Response {\n  // Set value to the session\n  let _ =\n    wisp_kv_sessions.set(\n      session_config,\n      req,\n      &quot;test_key&quot;,\n      &quot;something&quot;,\n      encode_string,\n    )\n  wisp.redirect(&quot;/&quot;)\n}\n```\n\n## Running the Example\n\nTo start the server, cd into /example and run `gleam run`.\n\nVisit (http://localhost:8000) in your browser. The page should display &quot;No value set. Go to /set to set a value.&quot;\n\n\nNavigate to (http://localhost:8000/set). You will be redirected back to the main page, which will now display &quot;something&quot;.\n\n# Development\n\n```sh\ngleam run   # Run the project\ngleam test  # Run the tests\njust watch-test # Run test and reload on file changes\n```\n\n# SessionStore Drivers\n\nA SessionStore driver is a function that produces a SessionStore instance. These drivers allow for custom session storage solutions by applying methods to the SessionStore object, potentially using external resources like databases.\n\nFor an example implementation, see ./src/memory_store.gleam.\n\n## Existing Drivers\n\n### memory_store (Included by Default)\nThe memory_store driver is suitable for development and testing purposes, but not recommended for production due to its non-concurrent nature. Internally, it uses an [Actor](https://hexdocs.pm/gleam_otp/gleam/otp/actor.html), which may become a bottleneck under heavy loads.\n\n*Usage Example:*\n\n```gleam\nuse memory_store &lt;- result.map(memory_store.try_create_session_store())\n```\n\n### postgress_store (In Development)\n\nA SessionStore driver for PostgreSQL is currently under development.\n","url":"index.html"},{"doc":"wisp_kv_sessions/session","title":"Expiry","content":"pub type Expiry {\n  ExpireAt(birl.Time)\n  ExpireIn(Int)\n}\n\nExpireAt(birl.Time)\n\n\nExpireIn(Int)\n\n\nSynonyms:\nwisp_kv_sessions/session.Expiry\nwisp_kv_sessions/session Expiry","url":"wisp_kv_sessions/session.html#Expiry"},{"doc":"wisp_kv_sessions/session","title":"Session","content":"pub type Session {\n  Session(\n    id: SessionId,\n    expires_at: #(#(Int, Int, Int), #(Int, Int, Int)),\n    data: dict.Dict(Key, JsonString),\n  )\n}\n\nSession(\n  id: SessionId,\n  expires_at: #(#(Int, Int, Int), #(Int, Int, Int)),\n  data: dict.Dict(Key, JsonString),\n)\n\n\nSynonyms:\nwisp_kv_sessions/session.Session\nwisp_kv_sessions/session Session","url":"wisp_kv_sessions/session.html#Session"},{"doc":"wisp_kv_sessions/session","title":"SessionBuilder","content":"pub type SessionBuilder {\n  SessionBuilder(\n    id: SessionId,\n    expiry: option.Option(Expiry),\n    data: dict.Dict(Key, JsonString),\n  )\n}\n\nSessionBuilder(\n  id: SessionId,\n  expiry: option.Option(Expiry),\n  data: dict.Dict(Key, JsonString),\n)\n\n\nSynonyms:\nwisp_kv_sessions/session.SessionBuilder\nwisp_kv_sessions/session SessionBuilder","url":"wisp_kv_sessions/session.html#SessionBuilder"},{"doc":"wisp_kv_sessions/session","title":"SessionError","content":"pub type SessionError {\n  UnknownError\n  DecodeError\n  DbSetupError\n  DbErrorInsertError(String)\n  DbErrorGetError(String)\n  DbErrorDeleteError(String)\n  DeserializeError(String)\n  SessionExpiredError\n  NoSessionError\n}\n\nUnknownError\n\n\nDecodeError\n\n\nDbSetupError\n\n\nDbErrorInsertError(String)\n\n\nDbErrorGetError(String)\n\n\nDbErrorDeleteError(String)\n\n\nDeserializeError(String)\n\n\nSessionExpiredError\n\n\nNoSessionError\n\n\nSynonyms:\nwisp_kv_sessions/session.SessionError\nwisp_kv_sessions/session SessionError","url":"wisp_kv_sessions/session.html#SessionError"},{"doc":"wisp_kv_sessions/session","title":"SessionId","content":"pub opaque type SessionId\n\n\nSynonyms:\nwisp_kv_sessions/session.SessionId\nwisp_kv_sessions/session SessionId","url":"wisp_kv_sessions/session.html#SessionId"},{"doc":"wisp_kv_sessions/session","title":"build","content":"pub fn build(session: SessionBuilder) -&gt; Session\n\nSynonyms:\nwisp_kv_sessions/session.build\nwisp_kv_sessions/session build","url":"wisp_kv_sessions/session.html#build"},{"doc":"wisp_kv_sessions/session","title":"builder","content":"pub fn builder() -&gt; SessionBuilder\n\nSynonyms:\nwisp_kv_sessions/session.builder\nwisp_kv_sessions/session builder","url":"wisp_kv_sessions/session.html#builder"},{"doc":"wisp_kv_sessions/session","title":"builder_from","content":"pub fn builder_from(session: Session) -&gt; SessionBuilder\n\nSynonyms:\nwisp_kv_sessions/session.builder_from\nwisp_kv_sessions/session builder_from","url":"wisp_kv_sessions/session.html#builder_from"},{"doc":"wisp_kv_sessions/session","title":"expiry_to_date","content":"pub fn expiry_to_date(\n  expiry: Expiry,\n) -&gt; #(#(Int, Int, Int), #(Int, Int, Int))\n Convert expiry type to a date\nSynonyms:\nwisp_kv_sessions/session.expiry_to_date\nwisp_kv_sessions/session expiry_to_date","url":"wisp_kv_sessions/session.html#expiry_to_date"},{"doc":"wisp_kv_sessions/session","title":"generate_id","content":"pub fn generate_id() -&gt; SessionId\n Create a new session id\nSynonyms:\nwisp_kv_sessions/session.generate_id\nwisp_kv_sessions/session generate_id","url":"wisp_kv_sessions/session.html#generate_id"},{"doc":"wisp_kv_sessions/session","title":"id_from_string","content":"pub fn id_from_string(id: String) -&gt; SessionId\n Unwrap session id to a string\nSynonyms:\nwisp_kv_sessions/session.id_from_string\nwisp_kv_sessions/session id_from_string","url":"wisp_kv_sessions/session.html#id_from_string"},{"doc":"wisp_kv_sessions/session","title":"id_to_string","content":"pub fn id_to_string(id: SessionId) -&gt; String\n Unwrap session id to a string\nSynonyms:\nwisp_kv_sessions/session.id_to_string\nwisp_kv_sessions/session id_to_string","url":"wisp_kv_sessions/session.html#id_to_string"},{"doc":"wisp_kv_sessions/session","title":"set_key_value","content":"pub fn set_key_value(\n  session: SessionBuilder,\n  key: String,\n  data: String,\n) -&gt; SessionBuilder\n\nSynonyms:\nwisp_kv_sessions/session.set_key_value\nwisp_kv_sessions/session set_key_value","url":"wisp_kv_sessions/session.html#set_key_value"},{"doc":"wisp_kv_sessions/session","title":"with_data","content":"pub fn with_data(\n  session: SessionBuilder,\n  data: Dict(String, String),\n) -&gt; SessionBuilder\n\nSynonyms:\nwisp_kv_sessions/session.with_data\nwisp_kv_sessions/session with_data","url":"wisp_kv_sessions/session.html#with_data"},{"doc":"wisp_kv_sessions/session","title":"with_expires_at","content":"pub fn with_expires_at(\n  session: SessionBuilder,\n  expires_at: Time,\n) -&gt; SessionBuilder\n\nSynonyms:\nwisp_kv_sessions/session.with_expires_at\nwisp_kv_sessions/session with_expires_at","url":"wisp_kv_sessions/session.html#with_expires_at"},{"doc":"wisp_kv_sessions/session","title":"with_expiry","content":"pub fn with_expiry(\n  session: SessionBuilder,\n  expiry: Expiry,\n) -&gt; SessionBuilder\n\nSynonyms:\nwisp_kv_sessions/session.with_expiry\nwisp_kv_sessions/session with_expiry","url":"wisp_kv_sessions/session.html#with_expiry"},{"doc":"wisp_kv_sessions/session","title":"with_id","content":"pub fn with_id(\n  session: SessionBuilder,\n  id: SessionId,\n) -&gt; SessionBuilder\n\nSynonyms:\nwisp_kv_sessions/session.with_id\nwisp_kv_sessions/session with_id","url":"wisp_kv_sessions/session.html#with_id"},{"doc":"wisp_kv_sessions/session","title":"with_id_string","content":"pub fn with_id_string(\n  session: SessionBuilder,\n  id: String,\n) -&gt; SessionBuilder\n\nSynonyms:\nwisp_kv_sessions/session.with_id_string\nwisp_kv_sessions/session with_id_string","url":"wisp_kv_sessions/session.html#with_id_string"},{"doc":"wisp_kv_sessions/session","title":"wisp_kv_sessions/session","content":"","url":"wisp_kv_sessions/session.html"},{"doc":"internal/utils","title":"get_session_id","content":"pub fn get_session_id(\n  cookie_name: String,\n  req: Request(Connection),\n) -&gt; Result(SessionId, SessionError)\n Get session Id from cookie\nSynonyms:\ninternal/utils.get_session_id\ninternal/utils get_session_id","url":"internal/utils.html#get_session_id"},{"doc":"internal/utils","title":"inject_session_cookie","content":"pub fn inject_session_cookie(\n  cookie_name cookie_name: String,\n  request req: Request(Connection),\n  value session_id: SessionId,\n  security security: Security,\n) -&gt; Request(Connection)\n Inject a cookie into a request replacing if it already exists\n This will NOT be persisted between requests\nSynonyms:\ninternal/utils.inject_session_cookie\ninternal/utils inject_session_cookie","url":"internal/utils.html#inject_session_cookie"},{"doc":"internal/utils","title":"is_session_expired","content":"pub fn is_session_expired(session: Session) -&gt; Bool\n\nSynonyms:\ninternal/utils.is_session_expired\ninternal/utils is_session_expired","url":"internal/utils.html#is_session_expired"},{"doc":"internal/utils","title":"remove_cookie","content":"pub fn remove_cookie(\n  req: Request(Connection),\n  name: String,\n) -&gt; Request(Connection)\n Remove a cookie from a request\n\n Remove a cookie from the request. If no cookie is found return the request unchanged.\nSynonyms:\ninternal/utils.remove_cookie\ninternal/utils remove_cookie","url":"internal/utils.html#remove_cookie"},{"doc":"internal/utils","title":"set_session_cookie","content":"pub fn set_session_cookie(\n  cookie_name: String,\n  response: Response(Body),\n  req: Request(Connection),\n  session: Session,\n) -&gt; Response(Body)\n Add session cookie to response \nSynonyms:\ninternal/utils.set_session_cookie\ninternal/utils set_session_cookie","url":"internal/utils.html#set_session_cookie"},{"doc":"internal/utils","title":"internal/utils","content":"","url":"internal/utils.html"},{"doc":"wisp_kv_sessions/session_config","title":"Config","content":"pub type Config {\n  Config(\n    default_expiry: session.Expiry,\n    cookie_name: String,\n    store: SessionStore,\n  )\n}\n\nConfig(\n  default_expiry: session.Expiry,\n  cookie_name: String,\n  store: SessionStore,\n)\n\n\nSynonyms:\nwisp_kv_sessions/session_config.Config\nwisp_kv_sessions/session_config Config","url":"wisp_kv_sessions/session_config.html#Config"},{"doc":"wisp_kv_sessions/session_config","title":"SessionStore","content":"pub type SessionStore {\n  SessionStore(\n    default_expiry: Int,\n    get_session: fn(session.SessionId) -&gt;\n      Result(\n        option.Option(session.Session),\n        session.SessionError,\n      ),\n    save_session: fn(session.Session) -&gt;\n      Result(session.Session, session.SessionError),\n    delete_session: fn(session.SessionId) -&gt;\n      Result(Nil, session.SessionError),\n  )\n}\n Session store is what the different storages should implement.\n It is used when saving and getting session\n\nSessionStore(\n  default_expiry: Int,\n  get_session: fn(session.SessionId) -&gt;\n    Result(option.Option(session.Session), session.SessionError),\n  save_session: fn(session.Session) -&gt;\n    Result(session.Session, session.SessionError),\n  delete_session: fn(session.SessionId) -&gt;\n    Result(Nil, session.SessionError),\n)\n\n\nSynonyms:\nwisp_kv_sessions/session_config.SessionStore\nwisp_kv_sessions/session_config SessionStore","url":"wisp_kv_sessions/session_config.html#SessionStore"},{"doc":"wisp_kv_sessions/session_config","title":"wisp_kv_sessions/session_config","content":"","url":"wisp_kv_sessions/session_config.html"},{"doc":"memory_store","title":"try_create_session_store","content":"pub fn try_create_session_store() -&gt; Result(\n  SessionStore,\n  SessionError,\n)\n\nSynonyms:\nmemory_store.try_create_session_store\nmemory_store try_create_session_store","url":"memory_store.html#try_create_session_store"},{"doc":"memory_store","title":"memory_store","content":"","url":"memory_store.html"},{"doc":"wisp_kv_sessions","title":"delete","content":"pub fn delete(\n  config: Config,\n  req: Request(Connection),\n  key: String,\n) -&gt; Result(Session, SessionError)\n Delete key from session\n\nSynonyms:\nwisp_kv_sessions.delete\nwisp_kv_sessions delete","url":"wisp_kv_sessions.html#delete"},{"doc":"wisp_kv_sessions","title":"delete_session","content":"pub fn delete_session(\n  config: Config,\n  req: Request(Connection),\n) -&gt; Result(Nil, SessionError)\n Remove session\n Usage: \n \n sessions.delete(store, req)\n \nSynonyms:\nwisp_kv_sessions.delete_session\nwisp_kv_sessions delete_session","url":"wisp_kv_sessions.html#delete_session"},{"doc":"wisp_kv_sessions","title":"get","content":"pub fn get(\n  config: Config,\n  req: Request(Connection),\n  key: String,\n  decoder: fn(Dynamic) -&gt; Result(a, List(DecodeError)),\n) -&gt; Result(Option(a), SessionError)\n Get data from session by key\n\nSynonyms:\nwisp_kv_sessions.get\nwisp_kv_sessions get","url":"wisp_kv_sessions.html#get"},{"doc":"wisp_kv_sessions","title":"get_session","content":"pub fn get_session(\n  config: Config,\n  req: Request(Connection),\n) -&gt; Result(Session, SessionError)\n Try to get the session from the store. \n If it does not exist create a new one.\n\nSynonyms:\nwisp_kv_sessions.get_session\nwisp_kv_sessions get_session","url":"wisp_kv_sessions.html#get_session"},{"doc":"wisp_kv_sessions","title":"middleware","content":"pub fn middleware(\n  config: Config,\n  req: Request(Connection),\n  handle_request: fn(Request(Connection)) -&gt; Response(Body),\n) -&gt; Response(Body)\n Create a session if no session exists to make sure there \n always is a session_id to store data towards\n Usage: \n \n use &lt;- sessions.middleware(config, req)\n \nSynonyms:\nwisp_kv_sessions.middleware\nwisp_kv_sessions middleware","url":"wisp_kv_sessions.html#middleware"},{"doc":"wisp_kv_sessions","title":"replace_session","content":"pub fn replace_session(\n  config: Config,\n  res: Response(Body),\n  req: Request(Connection),\n  new_session: Session,\n) -&gt; Result(Response(Body), SessionError)\n Replace the session with a new one\n Usage:\n \n wisp.ok() |&gt; replace_session(session_config, req)\n \n\nSynonyms:\nwisp_kv_sessions.replace_session\nwisp_kv_sessions replace_session","url":"wisp_kv_sessions.html#replace_session"},{"doc":"wisp_kv_sessions","title":"set","content":"pub fn set(\n  config: Config,\n  req: Request(Connection),\n  key: String,\n  data: a,\n  encoder: fn(a) -&gt; String,\n) -&gt; Result(a, SessionError)\n Set data in session by key\n\nSynonyms:\nwisp_kv_sessions.set\nwisp_kv_sessions set","url":"wisp_kv_sessions.html#set"},{"doc":"wisp_kv_sessions","title":"wisp_kv_sessions","content":"","url":"wisp_kv_sessions.html"}]);